@@ -139,6 +139,7 @@ search_keys_like_attributes (GList *modules, GckSession *session, GckAttributes
 		en = gck_modules_enumerate_objects (modules, search, GCK_SESSION_AUTHENTICATE | GCK_SESSION_READ_WRITE);
 
 		for (;;) {
+			gboolean done;
 			object = gck_enumerator_next (en, NULL, &error);
 			if (!object) {
 				if (error) {
@@ -148,7 +149,9 @@ search_keys_like_attributes (GList *modules, GckSession *session, GckAttributes
 				break;
 			}
 
-			if (!(func) (object, user_data))
+			done = !(func) (object, user_data);
+			g_object_unref (object);
+			if (done)
 				break;
 		}
 
@@ -435,6 +438,7 @@ remove_by_public_key (GckSession *session, GckObject *pub, gboolean exclude_v1)
 			g_free (label);
 			return;
 		}
+		g_free (label);
 	}
 
 	/* Lock token objects, remove session objects */
@@ -773,9 +777,10 @@ op_request_identities (GkdSshAgentCall *call)
 	g_return_val_if_fail (en, FALSE);
 
 	all_attrs = NULL;
-	do {
-		obj = gck_enumerator_next (en, NULL, &error);
-	} while (obj && load_identity_v2_attributes (obj, &all_attrs));
+	while ((obj = gck_enumerator_next (en, NULL, &error))) {
+		load_identity_v2_attributes (obj, &all_attrs);
+		g_object_unref (obj);
+	}
 
 	g_object_unref (en);
 
@@ -839,10 +844,10 @@ op_v1_request_identities (GkdSshAgentCall *call)
 	                                    GCK_SESSION_AUTHENTICATE | GCK_SESSION_READ_WRITE);
 
 	all_attrs = NULL;
-	do {
-		obj = gck_enumerator_next (en, NULL, &error);
-	} while (obj && load_identity_v1_attributes (obj, &all_attrs));
-
+	while ((obj = gck_enumerator_next (en, NULL, &error))) {
+		load_identity_v1_attributes (obj, &all_attrs);
+		g_object_unref (obj);
+	}
 	g_object_unref (en);
 
 	if (error) {
